#include <button_mlp101c.h>

#define BUTTONn  5

static void PBInit(Button_TypeDef Button, ButtonMode_TypeDef Button_Mode);
static uint8_t scan_button(void);

__IO uint16_t button_init_done = 0;
static uint16_t state_button = 0;
static uint8_t button_temp;
static uint8_t button_value = 0;

GPIO_TypeDef* BUTTON_PORT[BUTTONn] = {GPIOB, GPIOE, GPIOE, GPIOE, GPIOE}; 

const uint16_t BUTTON_PIN[BUTTONn] = {GPIO_Pin_9, GPIO_Pin_0, GPIO_Pin_1, GPIO_Pin_2, GPIO_Pin_3}; 

const uint32_t BUTTON_CLK[BUTTONn] = {RCC_APB2Periph_GPIOB,
                                      RCC_APB2Periph_GPIOE,
                                      RCC_APB2Periph_GPIOE,
                                      RCC_APB2Periph_GPIOE,
                                      RCC_APB2Periph_GPIOE};

const uint16_t BUTTON_EXTI_LINE[BUTTONn] = {EXTI_Line9, EXTI_Line0, EXTI_Line1, EXTI_Line2, EXTI_Line3 };

const uint16_t BUTTON_PORT_SOURCE[BUTTONn] = {GPIO_PortSourceGPIOB,
                                              GPIO_PortSourceGPIOE,
                                              GPIO_PortSourceGPIOE,
                                              GPIO_PortSourceGPIOE,
                                              GPIO_PortSourceGPIOE};
								 
const uint16_t BUTTON_PIN_SOURCE[BUTTONn] = {GPIO_PinSource9,
                                             GPIO_PinSource0,
                                             GPIO_PinSource1,
                                             GPIO_PinSource2,
                                             GPIO_PinSource3}; 

const uint16_t BUTTON_IRQn[BUTTONn] = {EXTI9_5_IRQn,
                                       EXTI0_IRQn,
                                       EXTI1_IRQn,
                                       EXTI2_IRQn,
                                       EXTI3_IRQn};


static void PBInit(Button_TypeDef Button, ButtonMode_TypeDef Button_Mode)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  EXTI_InitTypeDef EXTI_InitStructure;
  NVIC_InitTypeDef NVIC_InitStructure;

  /* Enable the BUTTON Clock */
  RCC_APB2PeriphClockCmd(BUTTON_CLK[Button] | RCC_APB2Periph_AFIO, ENABLE);

  /* Configure Button pin as input floating */
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_InitStructure.GPIO_Pin = BUTTON_PIN[Button];
  GPIO_Init(BUTTON_PORT[Button], &GPIO_InitStructure);


  if (Button_Mode == BUTTON_MODE_EXTI)
  {
    /* Connect Button EXTI Line to Button GPIO Pin */
    GPIO_EXTILineConfig(BUTTON_PORT_SOURCE[Button], BUTTON_PIN_SOURCE[Button]);

    /* Configure Button EXTI line */
    EXTI_InitStructure.EXTI_Line = BUTTON_EXTI_LINE[Button];
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;  
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init(&EXTI_InitStructure);

    /* Enable and set Button EXTI Interrupt to the lowest priority */
    NVIC_InitStructure.NVIC_IRQChannel = BUTTON_IRQn[Button];
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x0F;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x0F;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;

    NVIC_Init(&NVIC_InitStructure); 
  }
}

void button_init(void)
{
  PBInit(BUTTON_MODE, BUTTON_MODE_GPIO);
  PBInit(BUTTON_UP, BUTTON_MODE_GPIO);
  PBInit(BUTTON_DOWN, BUTTON_MODE_GPIO);
  PBInit(BUTTON_ENTER, BUTTON_MODE_GPIO);
  PBInit(BUTTON_ONOFF, BUTTON_MODE_GPIO);
  button_init_done = 1;
}

uint8_t PBGetState(Button_TypeDef Button)
{
  uint8_t data;
  if(GPIO_ReadInputDataBit(BUTTON_PORT[Button], BUTTON_PIN[Button]))
    data = 0;
  else
    data = 1;

  return data;
}

static uint8_t scan_button(void)
{
  return ((PBGetState(BUTTON_MODE) << 4)
         |(PBGetState(BUTTON_UP) << 3)
         |(PBGetState(BUTTON_DOWN) << 2)
         |(PBGetState(BUTTON_ENTER) << 1)
         |(PBGetState(BUTTON_ONOFF)));
}

void button_handler(void)
{
  switch(state_button)
  {
    case 0: //no button
      button_temp = scan_button();
      if(button_temp == 0)
        ;
      else
      {
        state_button = 1;
      }
      break;
    case 1:
      if(scan_button() == button_temp)
      {
        button_value = button_temp;
        state_button = 2;
      }
      else
        state_button = 0;
      break;
    case 2:
      if((scan_button() == 0) && (button_value == 0))
        state_button = 0;
      break;
    default:
      break;
  }
}

Button_TypeDef get_button(void)
{
  uint8_t temp = 0;
  if(button_value)
  {
    if(button_value & 0x10)
      temp = BUTTON_MODE;
    else if(button_value & 0x08)
      temp = BUTTON_UP;
    else if(button_value & 0x04)
      temp = BUTTON_DOWN;
    else if(button_value & 0x02)
      temp = BUTTON_ENTER;
    else if(button_value & 0x01)
      temp = BUTTON_ONOFF;

    temp++;
    button_value = 0;
  }
  return temp;
}

