/*
 * dac_mlp101c.c
 *
 *  Created on: 2015. 5. 3.
 *      Author: hcshin
 */

#include "dac_mlp101c.h"
#include "adc_mlp101c.h"
#include "utils.h"
#include <stdio.h>

static uint16_t dac_vol[4096];

static void dac_wr_config(uint8_t i2c_add, uint8_t data);

void dac_init(void)
{
  GPIO_InitTypeDef  GPIO_InitStructure;
  I2C_InitTypeDef  I2C_InitStructure;

  I2C_DeInit(I2C2);
  /* I2C1 clock enable */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C2, ENABLE);
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB | RCC_APB2Periph_AFIO, ENABLE);
  /* I2C2 SDA and SCL configuration - No Remapping */

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;  //SCL
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_OD;
  GPIO_Init(GPIOB, &GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11; //SDA
  GPIO_Init(GPIOB, &GPIO_InitStructure);

  /* I2C1 configuration */
  I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;
  I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_2;
  I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
  I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
  I2C_InitStructure.I2C_ClockSpeed = 400000;
  I2C_InitStructure.I2C_OwnAddress1 = 0x80;

  I2C_Init(I2C2, &I2C_InitStructure);
  //I2C_ITConfig(I2C1, I2C_IT_EVT|I2C_IT_ERR, ENABLE);
  I2C_Cmd(I2C2, DISABLE);
  I2C_Cmd(I2C2, ENABLE);
  dac_wr_config(0xc0, 0x90); //microchip
  //dac_wr_config(0xc4, 0x90);
  //dac_wr_data(0xc4, 0xfff);

}

static void dac_wr_config(uint8_t i2c_add, uint8_t data)
{
  while(I2C_GetFlagStatus(I2C2, I2C_FLAG_BUSY));
  /* Send START condition */
  I2C_GenerateSTART(I2C2, ENABLE);
  /* Test on EV5 and clear it */
  while (!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_MODE_SELECT));
  /* Send slave address for write */
  I2C_Send7bitAddress(I2C2, i2c_add, I2C_Direction_Transmitter);
  while (!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));
  I2C_SendData(I2C2, data);
  while (!I2C_GetFlagStatus(I2C2, I2C_FLAG_BTF));
  I2C_GenerateSTOP(I2C2, ENABLE);
  while (I2C_GetFlagStatus(I2C2, I2C_FLAG_STOPF));
}

void dac_wr_data(uint8_t i2c_add, uint16_t data)
{
  data &= 0xfff;
  while(I2C_GetFlagStatus(I2C2, I2C_FLAG_BUSY));
  /* Send START condition */
  I2C_GenerateSTART(I2C2, ENABLE);
  /* Test on EV5 and clear it */
  while (!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_MODE_SELECT));
  /* Send slave address for write */
  I2C_Send7bitAddress(I2C2, i2c_add, I2C_Direction_Transmitter);
  while (!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));
  I2C_SendData(I2C2, data >> 8);
  while (!I2C_GetFlagStatus(I2C2, I2C_FLAG_BTF));
  I2C_SendData(I2C2, data);
  while (!I2C_GetFlagStatus(I2C2, I2C_FLAG_BTF));
  I2C_GenerateSTOP(I2C2, ENABLE);
  while (I2C_GetFlagStatus(I2C2, I2C_FLAG_STOPF));
}

void dac_vol_set(uint16_t data)
{
  uint16_t i;
  for(i = 0; i < 4096; i++)
  {
    //if((dac_vol[i] / 2) > data)
    if((dac_vol[i]) >= (data * 2))
      break;
  }
  if(i == 4096)
    i = 4095;
  dac_wr_data(0xc0, i);
  //printf("dac:%d\r\n", i);
}

// not used in this version
// data format 0 -> 0A, 20 -> 2A
void dac_cur_set(uint8_t data)
{
  uint16_t dac;
  if(data == 0)
    data = 1; // 0.1A
  if(data > 20)
    data = 20; // 2A
  dac = 16380 / data;
  dac_wr_data(0xc4, dac + current0);
}

void dac_cal(void)
{
  uint16_t i;
  uint16_t j;
  uint16_t adc_value2;
  dac_wr_data(0xc0, 0);
  delay_100us(5000); //0.5s
  delay_100us(5000); //0.5s
  delay_100us(5000); //0.5s
  delay_100us(5000); //0.5s

  for(i = 0; i < 4096; i++)
  {
    adc_value2 = 0;
    dac_wr_data(0xc0, i);
    delay_100us(10);
    for(j = 0; j < 8; j++)
    {
      adc_value2 += ADC1ConvertedValue[2];
      delay_10us(2);
    }
    adc_value2 /= 8;
    dac_vol[i] = adc_value2;
  }
}

