// MPULAB
// MLP-101C REV.B
// 2015.11.23


/* Includes ------------------------------------------------------------------*/
#include "hw_config.h"
#include "usb_lib.h"
#include "usb_pwr.h"

#include "uart_mlp101c.h"
#include "led_mlp101c.h"
#include "lcd_mlp101c.h"
#include "button_mlp101c.h"
#include "buzzer_mlp101c.h"
#include "dac_mlp101c.h"
#include "io_mlp101c.h"
#include "adc_mlp101c.h"
#include "eeprom_mlp101c.h"
#include "utils.h"

#include <stdio.h>

/* Private typedef -----------------------------------------------------------*/
typedef enum {STATE_INIT = 0, STATE_NORMAL0, STATE_NORMAL1, STATE_USB, STATE_MAXCUR} STATE_MAIN;
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Extern variables ----------------------------------------------------------*/
__IO uint8_t PrevXferComplete = 1;

__IO uint16_t buttontick_cnt = 0;
//__IO uint16_t lcdtick_cnt = 0;
__IO uint16_t lcd_update = 0;

__IO uint32_t time_elapse = 0;

  int16_t current_value;
  uint16_t vol;
  uint16_t onoff = 0;
  float currentrat = 1.0;
  float mas = 0; //mA*s
  uint32_t power;
  uint16_t overcur = 0;

/* Private function prototypes -----------------------------------------------*/
void main_loop(void);
void calibration(void);
static void volttoa(uint16_t value, uint8_t *str);
static void curtoa(uint16_t value, uint8_t *str);
static void disp_volt(uint16_t value, uint8_t *str, uint16_t cur_position);
static void sectoa(uint32_t sec, uint8_t *str);
static void powertoa(uint32_t value, uint8_t *str);
static void mastoa(uint32_t value, uint8_t *str);

/* Private functions ---------------------------------------------------------*/

/*******************************************************************************
* Function Name  : main.
* Description    : main routine.
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
int main(void)
{
  uint32_t i;
  for(i = 0; i < 50000; i++);
  //printf("start\r\n");
  
  delay_init();
  led_init();
  
  if(SysTick_Config(SystemCoreClock / 50000)) //20us
  {
	while(1);
  }
  lcd_init();
  lcd_bl_init();
  lcd_write_str("MPULAB", 0, 0);
  delay_100us(5000);
  button_init();
  buzzer_init();
  buzzer(0, 1);
  dac_init();
  dac_wr_data(0xc0, 0);
  
  io_init();
  adc_init();
  eeprom_init();

  lcd_clear();
  if(PBGetState(BUTTON_MODE))
  {
    //printf("cal\r\n");
    calibration();
    while(1);
  }
  lcd_write_str("DAC Calibration", 0, 0);
  dac_cal();
  lcd_clear();
  lcd_write_str("Calibration Done", 0, 0);
  current0_cal();
  
  USB_Interrupts_Config();

  Set_USBClock();

  USB_Init();
  
  main_loop();

  while (1)
  {
  }
}

void main_loop(void)
{
  STATE_MAIN state_main = STATE_INIT;
  Button_TypeDef button_data;
  uint8_t str_volt[6];
  uint8_t str_cur[6];
  uint8_t str_time[11];
  uint8_t str_power[10];
  uint8_t str_mas[15];
  
  uint16_t cur_position = 0;
  uint32_t ftemp;
  uint16_t extradispmode = 0;
  
  while(1)
  {
    switch(state_main)
    {
      case STATE_INIT:
        lcd_clear();
        state_main = STATE_NORMAL0;
        vol = eeprom_read(EEADDVOL);
        if(vol > 2000)
          vol = 0;
        dac_vol_set(vol);
        if(eeprom_read(EEADDSIG) == RATSIGN)
        {
          ftemp = (eeprom_read(EEADDRATH) << 16) | eeprom_read(EEADDRATL);
          currentrat = *(float *)&ftemp;
        }
        else
          currentrat = 1.0;
        //printf("rat:%f\r\n", currentrat);
        onoff = 0;
        poweroff();

        break;
      case STATE_NORMAL0:
        lcd_clear();
        //lcd_write_str("Voltage Supply", 0, 0);
        lcd_write_str("V", 5, 1);
        lcd_write_str("A", 12, 1);
        disp_volt(vol, str_volt, cur_position);
        state_main = STATE_NORMAL1;
        break;
      case STATE_NORMAL1:
        button_data = get_button();
        switch(button_data)
        {
          case (BUTTON_MODE + 1):
            cur_position++;
            if(cur_position == 4)
              cur_position = 0;
            disp_volt(vol, str_volt, cur_position);
            break;
          case (BUTTON_UP + 1):
            switch(cur_position)
            {
              case 0: vol++; break;
              case 1: vol += 10; break;
              case 2: vol += 100; break;
              case 3: vol+= 1000; break;
            }
            if(vol > 2000)
              vol = 2000;
            disp_volt(vol, str_volt, cur_position);
            dac_vol_set(vol);
            if(onoff)
              eeprom_write(EEADDVOL, vol);
            break;
          case (BUTTON_DOWN + 1):
            switch(cur_position)
            {
              case 0: if(vol != 0)vol--; break;
              case 1: if(vol >= 10)vol -= 10; else vol = 0; break;
              case 2: if(vol >= 100)vol -= 100; else vol = 0; break;
              case 3: if(vol >= 1000)vol -= 1000; else vol = 0; break;
            }
            disp_volt(vol, str_volt, cur_position);
            dac_vol_set(vol);
            if(onoff)
              eeprom_write(EEADDVOL, vol);
            break;
          case (BUTTON_ONOFF + 1):
            if(onoff)
            {
              onoff = 0;
              poweroff();
              led_set(LED_OFF);
            }
            else
            {
              eeprom_write(EEADDVOL, vol);
              time_elapse = 0;
              mas = 0;
              onoff = 1;
              poweron();
              led_set(LED_GREEN);
            }
            break;
          case (BUTTON_ENTER + 1):
            extradispmode++;
            if(extradispmode == 3)
              extradispmode = 0;
            lcd_write_str("                ", 0, 0);
            break;
          default:
            break;
        }
        break;
      case STATE_USB:
        
        break;
    }
    if(state_main == STATE_NORMAL1) // normal mode current display
    {
      if(lcd_update)
      {
        lcd_update = 0;

        curtoa(current_value, str_cur);
        switch(extradispmode)
        {
          case 0: //elapse time
            sectoa(time_elapse, str_time);
            break;
          case 1: //power
            powertoa(power, str_power);
            break;
        }
        lcd_curoff();
        lcd_write_str(str_cur, 7, 1);
        switch(extradispmode)
        {
          case 0:
            lcd_write_str(str_time, 0, 0);
            break;
          case 1:
            lcd_write_str(str_power, 0, 0);
            break;
          case 2:
            mastoa((uint32_t)(mas * 10000), str_mas);
            lcd_write_str(str_mas, 0, 0);
            break;
        }
        lcd_curon();
        
        disp_volt(vol, str_volt, cur_position);
      }
    }
    
    // current max limit
    if(state_main != STATE_USB)
    {

      //if(current_value > 2000)
      if(overcur)
      {
        //onoff = 0;
        //poweroff();
        overcur = 0;
        led_set(LED_RED);
        lcd_clear();
        lcd_write_str("CURRENT LIMIT", 0, 0);
        buzzer(1, 1);
        delay_100us(5000);
        delay_100us(5000);
        led_set(LED_OFF);
        if(state_main != STATE_USB)
          state_main = STATE_NORMAL0;
        else
        {
          lcd_curoff();
          lcd_clear();
          lcd_write_str("USB CONNECTED", 0, 0);
        }
      }
    }
    
    if(state_main != STATE_USB)
    {
      if(bDeviceState == CONFIGURED)
      {
        poweroff();
        vol = 0;
        dac_vol_set(vol);
        led_set(LED_OFF);
        lcd_curoff();
        lcd_clear();
        lcd_write_str("USB CONNECTED", 0, 0);
        state_main = STATE_USB;
      }
    }
  }
}

static void volttoa(uint16_t value, uint8_t *str)
{
  uint16_t tempvalue;
  
  tempvalue = value / 1000;
  value = value - (tempvalue * 1000);
  *str++ = tempvalue + 0x30;
  tempvalue = value / 100;
  value = value - (tempvalue * 100);
  *str++ = tempvalue + 0x30;
  *str++ = '.';
  tempvalue = value / 10;
  value = value - (tempvalue * 10);
  *str++ = tempvalue + 0x30;
  *str++ = value + 0x30;
  *str = 0;
}

static void curtoa(uint16_t value, uint8_t *str)
{
  uint16_t tempvalue;
  
  tempvalue = value / 1000;
  value = value - (tempvalue * 1000);
  *str++ = tempvalue + 0x30;
  *str++ = '.';
  tempvalue = value / 100;
  value = value - (tempvalue * 100);
  *str++ = tempvalue + 0x30;
  tempvalue = value / 10;
  value = value - (tempvalue * 10);
  *str++ = tempvalue + 0x30;
  *str++ = value + 0x30;
  *str = 0;
}

static void sectoa(uint32_t sec, uint8_t *str)
{
  uint32_t temptime, temptime2;
                          
  temptime = sec / 3600; // hour
  sec = sec - (temptime * 3600);
  
  temptime2 = temptime / 1000;
  temptime = temptime - (temptime2 * 1000);
  *str++ = temptime2 + 0x30;
  temptime2 = temptime / 100;
  temptime = temptime - (temptime2 * 100);
  *str++ = temptime2 + 0x30;
  temptime2 = temptime / 10;
  temptime = temptime - (temptime2 * 10);
  *str++ = temptime2 + 0x30;
  *str++ = temptime + 0x30;
  *str++ = ':';
  
  temptime = sec / 60; //minute
  sec = sec - (temptime * 60);
  temptime2 = temptime / 10;
  temptime = temptime - (temptime2 * 10);
  *str++ = temptime2 + 0x30;
  *str++ = temptime + 0x30;
  *str++ = ':';
  
  temptime = sec;
  temptime2 = temptime / 10;
  temptime = temptime - (temptime2 * 10);
  *str++ = temptime2 + 0x30;
  *str++ = temptime + 0x30;
  *str = 0;
}

static void powertoa(uint32_t value, uint8_t *str)
{
  uint32_t tempvalue;
  
  tempvalue = value / 1000000;
  value =  value - (tempvalue * 1000000);
  *str++ = tempvalue + 0x30;
  tempvalue = value / 100000;
  value =  value - (tempvalue * 100000);
  *str++ = tempvalue + 0x30;
  *str++ = '.';
  tempvalue = value / 10000;
  value =  value - (tempvalue * 10000);
  *str++ = tempvalue + 0x30;
  tempvalue = value / 1000;
  value =  value - (tempvalue * 1000);
  *str++ = tempvalue + 0x30;
  tempvalue = value / 100;
  value = value - (tempvalue * 100);
  *str++ = tempvalue + 0x30;
  tempvalue = value / 10;
  value = value - (tempvalue * 10);
  *str++ = tempvalue + 0x30;
  *str++ = value + 0x30;
  *str++ = 'W';
  *str = 0;
}

static void mastoa(uint32_t value, uint8_t *str)
{
  uint32_t tempvalue;
  
  tempvalue = value / 10000000;
  value =  value - (tempvalue * 10000000);
  *str++ = tempvalue + 0x30;
  tempvalue = value / 1000000;
  value =  value - (tempvalue * 1000000);
  *str++ = tempvalue + 0x30;
  tempvalue = value / 100000;
  value =  value - (tempvalue * 100000);
  *str++ = tempvalue + 0x30;
  tempvalue = value / 10000;
  value =  value - (tempvalue * 10000);
  *str++ = tempvalue + 0x30;
  *str++ = '.';
  tempvalue = value / 1000;
  value =  value - (tempvalue * 1000);
  *str++ = tempvalue + 0x30;
  tempvalue = value / 100;
  value = value - (tempvalue * 100);
  *str++ = tempvalue + 0x30;
  tempvalue = value / 10;
  value = value - (tempvalue * 10);
  *str++ = tempvalue + 0x30;
  *str++ = value + 0x30;
  *str++ = 'm';
  *str++ = 'A';
  *str++ = 'h';
  *str = 0;
}

static void disp_volt(uint16_t value, uint8_t *str, uint16_t cur_position)
{
  volttoa(value, str);
  lcd_curoff();
  lcd_write_str(str, 0, 1);
  if(cur_position == 0)
  {
    lcd_curleft();
  }
  else if(cur_position == 1)
  {
    lcd_curleft();
    lcd_curleft();
  }
  else if(cur_position == 2)
  {
    lcd_curleft();
    lcd_curleft();
    lcd_curleft();
    lcd_curleft();
  }
  else if(cur_position == 3)
  {
    lcd_curleft();
    lcd_curleft();
    lcd_curleft();
    lcd_curleft();
    lcd_curleft();
  }
  lcd_curon();
}

void calibration(void)
{
  uint16_t value, tempvalue;
  uint8_t str[10];
  
  get_button();
  get_button();
  dac_wr_data(0xc0, 0xfff);
  lcd_write_str("Adj VR1 to 20V", 0, 0);
  lcd_write_str("get Volt at TP14", 0, 1);
  
  while(PBGetState(BUTTON_MODE) == 1);
  while(PBGetState(BUTTON_MODE) == 0);
  while(PBGetState(BUTTON_MODE) == 1);
  lcd_clear();
  lcd_write_str("Adj VR2 to 4000", 0, 0);
  while(1)
  {
    delay_100us(1000);
    value = adc_ch2_value;
    tempvalue = value / 1000;
    value = value - (tempvalue * 1000);
    str[0] = tempvalue + 0x30;
    tempvalue = value / 100;
    value = value - (tempvalue * 100);
    str[1] = tempvalue + 0x30;
    tempvalue = value / 10;
    value = value - (tempvalue * 10);
    str[2] = tempvalue + 0x30;
    str[3] = value + 0x30;
    str[4] = 0;
    lcd_write_str(str, 0, 1);
  }
}

#ifdef  USE_FULL_ASSERT
/*******************************************************************************
* Function Name  : assert_failed
* Description    : Reports the name of the source file and the source line number
*                  where the assert_param error has occurred.
* Input          : - file: pointer to the source file name
*                  - line: assert_param error line source number
* Output         : None
* Return         : None
*******************************************************************************/
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while(1)
  {
  }
}
#endif

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
